/*===========================================================================*\

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef SPHERECOLLIDABLE_H
#define SPHERECOLLIDABLE_H

#include "Collidable.h"

class Moveable;

/**
 * Pretty much what it sounds like - this is a spherical collidable.
 */
class SphereCollidable : public Collidable {
   private:
      Moveable *moveable;
      Float radius;

   public:
      static Float getPointLineDistanceSquared(const Vector &point, 
                                        const Vector &linePoint,
                                        const Vector &lineVector
                                        );

      /**
       * Returns the distance d to the point on the line defined by org & dir
       * (the "subject line") such that org + d * dir is of distance "dist"
       * from the line defined by linePoint & lineVector (the "target line").
       * This is used to calculate the point that a sphere collides with the
       * edge of a polygon.
       * @param linePoint a point on the target line
       * @param lineVector the vector of the target line
       * @param org a point on the subject line
       * @param distSquared 
       *    the square of the desired distance between the point on the
       *    subject line and the target line squared.  We expect a square as a
       *    small optimization because we would have to square it anyway and
       *    most contexts where it is used should have already done this.
       * @param offset 
       *    the result.  "offset * dir + org" is of distance sqrt(distSquared)
       *    from the target line.
       * @return false if there is no point on the line of the specified
       *         distance
       */
      static bool getLinePointAtDistance(const Vector &linePoint,
                                         const Vector &lineVector,
                                         const Vector &org,
                                         const Vector &dir,
                                         Float distSquared,
                                         Float &offset
                                         );
   private:
      bool getPlaneIntersection(const Vector &org, const Vector &dir, 
                                const Vector &normal,
                                const Vector &pointOnPlane,
                                Vector &intersection,
                                Float &distance
                                ) const;
      bool checkForPolyCollision(const Poly &polygon, const Vector &dir,
                                 Float &distance
                                 ) const;
   public:
      SphereCollidable(Moveable *moveable, Float radius);

      virtual Types getTypes() const;
      virtual int getPriority() const;
      virtual void informOfCollision(const CollidablePtr &other,
                                     Float distance
                                     );
      virtual const Poly &getPoly() const;
      virtual AxisBox getAxisBox() const;
      virtual Sphere getSphere() const;
      virtual Vector getVelocity() const;
      virtual void setMoveable(Moveable *moveable);
      virtual bool checkForCollision(const CollidablePtr &other, 
                                     Float &time) const;
};

#endif
