#==============================================================================
#
#  $Id$
#
"""
   Tools to help debug wurld.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import wurld
import time
from util import vecToStr

_engine = wurld.getGameEngine()

class TeleportTracer:
   """
      Instances of this can be used to track objects for "wacky teleportation"
      problems that seem to show up.  Specifically, an instance of this class
      keeps track of when an object moves a distance greater than it should
      have given its velocity.
   """

   def __init__(self, moveable):
      self.last = moveable.getPosition()
      self.lastTime = time.time()
      self.moveable = moveable
      self.obj = wurld.createPythonObject()
      self.obj.setRenderFrameCallback(self)
      _engine.addGameObject(self.obj)

   def __call__(self, engine, object):
      t = time.time()
      timeDiff = t - self.lastTime
      pos = self.moveable.getPosition()
      if (pos - self.last).size() >  \
         self.moveable.getVelocity().size() * timeDiff * 2:
         print 'teleported from %s to %s' % \
            (vecToStr(self.last), vecToStr(pos))
      self.last = pos
      self.lastTime = t

