#==============================================================================
#
#  $Id$
#
"""
   Creates a moving pyramid with a flattened top.
"""
#
#  Copyright (C) 2006 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import wurld

def makeQuadramidModel():
   model = wurld.createModel()
   v = wurld.createVector
   q = wurld.createQuad

   blb = v(-2.5, 0, -2.5) *2
   blf = v(-2.5, 0, 2.5) *2
   brb = v(2.5, 0, -2.5) *2
   brf = v(2.5, 0, 2.5) *2
   tlb = v(-1, 5, -1) *2
   tlf = v(-1, 5, 1) *2
   trb = v(1, 5, -1) *2
   trf = v(1, 5, 1) *2
   stone = wurld.createTextureMaterial(wurld.createTexture('stucco.png'), 10, 10)

   # bottom
   model.addPoly(q(blf, blb, brb, brf, stone))

   # front
   model.addPoly(q(trf, tlf, blf, brf, stone))

   # back
   model.addPoly(q(tlb, trb, brb, blb, stone))

   # left 
   model.addPoly(q(tlf, tlb, blb, blf, stone))

   # right
   model.addPoly(q(trb, trf, brf, brb, stone))

   # top
   model.addPoly(q(trb, tlb, tlf, trf, stone))

   return model

class Quadramid:
   def onEngineFrame(self, engine, pythonObj):

      pos = self.moveable.getPosition()
      vel = self.moveable.getVelocity()
      changeVelocity = False
      if pos.y < 0 or pos.y > 20 and vel.y >= 0:
         vel.y = vel.y * -1
         changeVelocity = True

      if changeVelocity:
         self.moveable.setVelocity(vel)

   def onCollision(self):
      print 'Quadramid has been hit'
      vel = self.moveable.getVelocity()
      vel.x = vel.x * -1
      vel.y = vel.y * -1
      vel.z = vel.z * -1
      self.moveable.setVelocity(vel)

   def __init__(self):
      model = makeQuadramidModel()
      self.obj = wurld.createPythonObject()
      self.moveable = wurld.createMoveable()
      self.moveable.setModel(model)
      self.moveable.setPosition(wurld.createVector(20, 10, 20))
      self.moveable.setVelocity(wurld.createVector(10, 10, 10))

      self.obj.setMoveable(self.moveable)
      wurld.getGameEngine().addGameObject(self.obj)
      wurld.getGameEngine().addRenderableObject(self.obj)
      self.obj.setEngineFrameCallback(self.onEngineFrame)

      # set up the collision stuff - use an axis box collision model and the point
      # collision algo
      collidable = wurld.createSimpleCollidable(model.getBoundingBox())
      self.moveable.addCollidable(collidable)
      wurld.getGameEngine().addCollidable(collidable)
      self.moveable.setCollisionAlgo(wurld.createPointCollisionAlgo())
      self.obj.setCollisionCallback(self.onCollision)

q = Quadramid()
